@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Tk::HashEditor;
-$Config::Model::Tk::HashEditor::VERSION = '1.341';
+$Config::Model::Tk::HashEditor::VERSION = '1.342';
 use strict;
 use warnings;
 use Carp;
@@ -303,7 +303,7 @@ sub add_entry {
         $logger->debug("add_entry on ordered hash: swap $idx and $add");
         $hash->move_after( $add, $idx );
         $logger->debug(
-            "new hash idx: " . join( ',', $hash->get_all_indexes ) );
+            "new hash idx: " . join( ',', $hash->fetch_all_indexes ) );
         my $new_idx = $selected[0] + 1;
         $tklist->insert( $new_idx, $add );
         $tklist->selectionSet($new_idx);
@@ -477,7 +477,7 @@ sub move_selected_up {
     my $hash = $cw->{hash};
     $hash->move_up($name);
     $logger->debug(
-        "move_up new hash idx: " . join( ',', $hash->get_all_indexes ) );
+        "move_up new hash idx: " . join( ',', $hash->fetch_all_indexes ) );
 
     $cw->reload_tree;
 }
@@ -487,7 +487,7 @@ sub move_selected_down {
     my $tklist = $cw->Subwidget('tklist');
     my @idx    = $tklist->curselection();
     my $hash   = $cw->{hash};
-    my @h_idx  = $hash->get_all_indexes;
+    my @h_idx  = $hash->fetch_all_indexes;
 
     return unless @idx and $idx[0] < $#h_idx;
 
@@ -502,7 +502,7 @@ sub move_selected_down {
 
     $hash->move_down($name);
     $logger->debug(
-        "move_down new hash idx: " . join( ',', $hash->get_all_indexes ) );
+        "move_down new hash idx: " . join( ',', $hash->fetch_all_indexes ) );
 
     $cw->reload_tree;
 }
