@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Tk::LeafEditor ;
-$Config::Model::Tk::LeafEditor::VERSION = '1.341';
+$Config::Model::Tk::LeafEditor::VERSION = '1.342';
 use strict;
 use warnings ;
 use Carp ;
@@ -250,31 +250,19 @@ sub store {
 
     print "Storing '$v'\n";
 
-    $cw->{leaf}->store(
-        value => $v,
-        callback => sub { $cw->store_cb(@_) ;},
-    );
+    eval {$cw->{leaf}->store($v); } ;
 
-}
-
-sub store_cb {
-    my $cw = shift;
-    my %args = @_ ;
-
-    my ($value, $check, $silent, $notify_change, $ok, $callback)
-        = @args{qw/value check silent notify_change ok callback/} ;
-
-    if ($ok) {
+    if ($@) {
+        $cw -> Dialog ( -title => 'Value error',
+                        -text  => $@->as_string,
+                      )
+            -> Show ;
+    }
+    else {
         # trigger redraw of Tk Tree
         $cw->{store_cb}->() ;
         $cw->update_warning($cw->{leaf}) ;
     }
-    else {
-        $cw -> Dialog (
-            -title => 'Value error',
-            -text  => $cw->{leaf}->error_msg,
-        ) -> Show ;
-    }
 }
 
 sub set_value_help {
